"""
Models and Enums for the database.
All table classes should be suffixed with `Model`.
"""

from sqlalchemy.sql import func
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    BigInteger,
    UniqueConstraint,
    ForeignKey
)

Base = declarative_base()


class AuditModel(Base):
    """
    Table for taking audits.
    """

    __tablename__ = "audit"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_user_id = Column(BigInteger, nullable=False)
    message = Column(String, nullable=False)
    created = Column(DateTime(timezone=True), server_default=func.now(), nullable=False)  # pylint: disable=E1102


class SentArticleModel(Base):
    """
    Table for tracking articles that have been served by the bot.
    """

    __tablename__ = "sent_articles"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_message_id = Column(BigInteger, nullable=False)
    discord_channel_id = Column(BigInteger, nullable=False)
    discord_server_id = Column(BigInteger, nullable=False)
    article_url = Column(String, nullable=False)
    when = Column(DateTime(timezone=True), server_default=func.now(), nullable=False)  # pylint: disable=E1102


class RssSourceModel(Base):
    """
    Table for user submitted news feeds.
    """

    __tablename__ = "rss_source"

    id = Column(Integer, primary_key=True, autoincrement=True)
    nick = Column(String, nullable=False)
    discord_server_id = Column(BigInteger, nullable=False)
    rss_url = Column(String, nullable=False)
    created = Column(DateTime(timezone=True), server_default=func.now(), nullable=False)  # pylint: disable=E1102

    feed_channels = relationship("FeedChannelModel", cascade="all, delete")

    # the nickname must be unique, but only within the same discord server
    __table_args__ = (
        UniqueConstraint('nick', 'discord_server_id', name='uq_nick_discord_server'),
    )


class FeedChannelModel(Base):
    """
    Table representing discord channels to be used for news feeds.
    """

    __tablename__ = "feed_channel"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_channel_id = Column(BigInteger, nullable=False)
    discord_server_id = Column(BigInteger, nullable=False)
    search_name = Column(String, nullable=False)
    rss_source_id = Column(Integer, ForeignKey('rss_source.id'), nullable=False)

    rss_source = relationship("RssSourceModel", overlaps="feed_channels", lazy="joined")

    # the rss source must be unique, but only within the same discord channel
    __table_args__ = (
        UniqueConstraint('rss_source_id', 'discord_channel_id', name='uq_rss_discord_channel'),
    )
